@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -703,7 +703,7 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
 
       return TRUE;
     }
-  else if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS) && event->button == 2)
+  else if (event->type == GDK_BUTTON_PRESS && event->button == 2)
     {
       /* determine the path to the item that was middle-clicked */
       if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL))
@@ -713,44 +713,30 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
           gtk_tree_selection_unselect_all (selection);
           gtk_tree_selection_select_path (selection, path);
 
-          /* if the event was a double-click or we are in single-click mode, then
-           * we'll open the file or folder (folder's are opened in new windows)
-           */
-          if (G_LIKELY (event->type == GDK_2BUTTON_PRESS || exo_tree_view_get_single_click (EXO_TREE_VIEW (tree_view))))
+          /* determine the file for the path */
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (details_view)->model), &iter, path);
+          file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (details_view)->model, &iter);
+          if (G_LIKELY (file != NULL) && thunar_file_is_directory (file))
             {
-              /* determine the file for the path */
-              gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (details_view)->model), &iter, path);
-              file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (details_view)->model, &iter);
-              if (G_LIKELY (file != NULL))
-                {
-                  /* determine the action to perform depending on the type of the file */
-                  if (thunar_file_is_directory (file))
-                    {
-                      /* lookup setting if we should open in a tab or a window */
-                      preferences = thunar_preferences_get ();
-                      g_object_get (preferences, "misc-middle-click-in-tab", &in_tab, NULL);
-                      g_object_unref (preferences);
+              /* lookup setting if we should open in a tab or a window */
+              preferences = thunar_preferences_get ();
+              g_object_get (preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+              g_object_unref (preferences);
 
-                      /* holding ctrl inverts the action */
-                      if ((event->state & GDK_CONTROL_MASK) != 0)
-                        in_tab = !in_tab;
+              /* holding ctrl inverts the action */
+              if ((event->state & GDK_CONTROL_MASK) != 0)
+                  in_tab = !in_tab;
 
-                      action_name = in_tab ? "open-in-new-tab" : "open-in-new-window";
-                    }
-                  else
-                    {
-                      action_name = "open";
-                    }
+              action_name = in_tab ? "open-in-new-tab" : "open-in-new-window";
 
-                  action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, action_name);
-      
-                  /* emit the action */
-                  if (G_LIKELY (action != NULL))
-                    gtk_action_activate (action);
+              action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (details_view)->ui_manager, action_name);
 
-                  /* release the file reference */
-                  g_object_unref (G_OBJECT (file));
-                }
+              /* emit the action */
+              if (G_LIKELY (action != NULL))
+                  gtk_action_activate (action);
+
+              /* release the file reference */
+              g_object_unref (G_OBJECT (file));
             }
 
           /* cleanup */
