@@ -97,6 +97,8 @@ static gboolean                 thunar_tree_view_button_press_event           (G
                                                                                GdkEventButton          *event);
 static gboolean                 thunar_tree_view_button_release_event         (GtkWidget               *widget,
                                                                                GdkEventButton          *event);
+static gboolean                 thunar_tree_view_key_press_event              (GtkWidget               *widget,
+                                                                               GdkEventKey             *event);
 static void                     thunar_tree_view_drag_data_received           (GtkWidget               *widget,
                                                                                GdkDragContext          *context,
                                                                                gint                     x,
@@ -432,6 +434,9 @@ thunar_tree_view_init (ThunarTreeView *view)
   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
   gtk_tree_selection_set_select_function (selection, thunar_tree_view_selection_func, view, NULL);
 
+  /* custom keyboard handler for better navigation */
+  g_signal_connect (GTK_WIDGET (view), "key_press_event", G_CALLBACK (thunar_tree_view_key_press_event), NULL);
+
   /* enable drop support for the tree view */
   gtk_drag_dest_set (GTK_WIDGET (view), 0, drop_targets, G_N_ELEMENTS (drop_targets),
                      GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
@@ -683,10 +688,11 @@ static gboolean
 thunar_tree_view_button_press_event (GtkWidget      *widget,
                                      GdkEventButton *event)
 {
-  ThunarTreeView *view = THUNAR_TREE_VIEW (widget);
-  GtkTreePath    *path;
-  GtkTreeIter     iter;
-  gboolean        result;
+  ThunarTreeView    *view = THUNAR_TREE_VIEW (widget);
+  GtkTreeViewColumn *column;
+  GtkTreePath       *path;
+  GtkTreeIter        iter;
+  gboolean           result;
 
   /* reset the pressed button state */
   view->pressed_button = -1;
@@ -699,7 +705,7 @@ thunar_tree_view_button_press_event (GtkWidget      *widget,
   result = (*GTK_WIDGET_CLASS (thunar_tree_view_parent_class)->button_press_event) (widget, event);
 
   /* resolve the path at the cursor position */
-  if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL))
+  if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, &column, NULL, NULL))
     {
       /* check if we should popup the context menu */
       if (G_UNLIKELY (event->button == 3 && event->type == GDK_BUTTON_PRESS))
@@ -715,9 +721,15 @@ thunar_tree_view_button_press_event (GtkWidget      *widget,
             }
         }
       else if ((event->button == 1 || event->button == 2)
-               && event->type == GDK_BUTTON_PRESS
-               && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
+               && event->type == GDK_BUTTON_PRESS)
         {
+          GdkRectangle rect;
+          gtk_tree_view_get_cell_area (GTK_TREE_VIEW (widget), path, column, &rect);
+
+          /* set cursor only when the user did not click the expander */
+          if (rect.x <= event->x && event->x <= (rect.x + rect.width))
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
+
           /* remember the button as pressed and handled it in the release handler */
           view->pressed_button = event->button;
         }
@@ -749,6 +761,10 @@ thunar_tree_view_button_release_event (GtkWidget      *widget,
       else if (G_UNLIKELY (event->button == 2))
         {
           g_object_get (view->preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+          /* holding ctrl inverts the action */
+          if ((event->state & GDK_CONTROL_MASK) != 0)
+            in_tab = !in_tab;
+
           if (in_tab)
             thunar_tree_view_action_open_in_new_tab (view);
           else
@@ -763,6 +779,106 @@ thunar_tree_view_button_release_event (GtkWidget      *widget,
   return (*GTK_WIDGET_CLASS (thunar_tree_view_parent_class)->button_release_event) (widget, event);
 }
 
+static gboolean
+thunar_tree_view_key_press_event(GtkWidget   *widget,
+                                 GdkEventKey *event)
+{
+  ThunarTreeView *tree_view = THUNAR_TREE_VIEW (widget);
+  gboolean stopPropagation = FALSE;
+  GtkTreePath *path;
+
+  /* Get path of currently highlighted item */
+  gtk_tree_view_get_cursor(GTK_TREE_VIEW (tree_view), &path, NULL);
+
+  switch (event->keyval)
+    {
+    case GDK_KEY_Up:
+    case GDK_KP_Up:
+    case GDK_KEY_Down:
+    case GDK_KP_Down:
+      /* the default actions works good, but we want to update the right pane */
+      GTK_WIDGET_CLASS (thunar_tree_view_parent_class)->key_press_event (widget, event);
+
+      /* sync with new tree view selection */
+      gtk_tree_path_free (path);
+      gtk_tree_view_get_cursor (GTK_TREE_VIEW (tree_view), &path, NULL);
+
+      /* The top-level desktop entry is a bit problematic; opening it
+         will usually cause expansion of the home directory where the
+         desktop folder resides. This breaks navigation and is bad for
+         user experience, so do not open the top-level desktop entry
+         automatically.
+       */
+      if (gtk_tree_path_get_depth (path) == 1)
+        {
+          ThunarFile *file;
+          GFile *desktop;
+          GFile *selected;
+
+          file = thunar_tree_view_get_selected_file (tree_view);
+          if (file)
+            {
+              desktop = thunar_g_file_new_for_desktop ();
+              selected = thunar_file_get_file (file);
+
+              /* open only non-desktop top-level items */
+              if (! g_file_equal (desktop, selected))
+                  thunar_tree_view_action_open (tree_view);
+
+              g_object_unref (G_OBJECT (file));
+              g_object_unref (desktop);
+            }
+        }
+      else
+          thunar_tree_view_action_open (tree_view);
+
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_Left:
+    case GDK_KP_Left:
+      /* if branch is expanded then collapse it */
+      if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
+          gtk_tree_view_collapse_row (GTK_TREE_VIEW (tree_view), path);
+
+      else /* if branch is already collapsed then move to parent */
+          if (gtk_tree_path_up (path))
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, NULL, FALSE);
+      thunar_tree_view_action_open (tree_view);
+
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_KEY_Right:
+    case GDK_KP_Right:
+      /* if branch is not expanded then expand it */
+      if (!gtk_tree_view_row_expanded (GTK_TREE_VIEW (tree_view), path))
+          gtk_tree_view_expand_row (GTK_TREE_VIEW (tree_view), path, FALSE);
+
+      else /* if branch is already expanded then move to first child */
+        {
+          gtk_tree_path_down (path);
+          gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, NULL, FALSE);
+          thunar_tree_view_action_open (tree_view);
+        }
+
+      stopPropagation = TRUE;
+      break;
+
+    case GDK_space:
+    case GDK_Return:
+    case GDK_KP_Enter:
+      thunar_tree_view_open_selection (tree_view);
+      stopPropagation = TRUE;
+      break;
+    }
+
+  gtk_tree_path_free (path);
+  if (stopPropagation)
+      gtk_widget_grab_focus (widget);
+
+  return stopPropagation;
+}
 
 
 static void
@@ -1093,6 +1209,7 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
   GtkWidget    *menu;
   GtkWidget    *item;
   GtkWidget    *window;
+  GIcon        *icon;
   GList        *providers, *lp;
   GList        *actions = NULL, *tmp;
 
@@ -1201,6 +1318,12 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
       g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_create_folder), view);
       gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
       gtk_widget_show (item);
+      
+      /* set the stock icon */
+      icon = g_themed_icon_new ("folder-new");
+      image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+      g_object_unref (icon);
 
       /* append a separator item */
       item = gtk_separator_menu_item_new ();
@@ -2276,6 +2399,7 @@ thunar_tree_view_new_files (ThunarJob      *job,
     {
       /* change to the newly created folder */
       thunar_navigator_change_directory (THUNAR_NAVIGATOR (view), file);
+      g_object_unref (file);
     }
 }
 
